@@ -599,6 +599,8 @@ static inline char *ReplaceSpaceWithNewline(char **caption,char *space)
   size_t
     octets;
 
+  if ((caption == (char **) NULL) || (space == (char *) NULL))
+    return((char *) NULL);
   octets=(size_t) GetUTFOctets(space);
   if (octets == 1)
     *space='\n';
@@ -628,14 +630,14 @@ MagickExport ssize_t FormatMagickCaption(Image *image,DrawInfo *draw_info,
   const MagickBooleanType split,TypeMetric *metrics,char **caption,
   ExceptionInfo *exception)
 {
-  MagickBooleanType
-    status;
-
   char
     *p,
     *q,
     *s;
 
+  MagickBooleanType
+    status;
+
   size_t
     width;
 
@@ -985,9 +987,6 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
   DrawInfo
     *annotate_info;
 
-  ExceptionInfo
-    *sans_exception;
-
   MagickBooleanType
     status;
 
@@ -1070,19 +1069,18 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
         }
       font=DestroyString(font);
     }
-  sans_exception=AcquireExceptionInfo();
   if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily("Open Sans",draw_info->style,
-      draw_info->stretch,draw_info->weight,sans_exception);
-  if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily("Sans Serif",draw_info->style,
-      draw_info->stretch,draw_info->weight,sans_exception);
-  if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily((const char *) NULL,draw_info->style,
-      draw_info->stretch,draw_info->weight,sans_exception);
-  if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfo("*",sans_exception);
-  sans_exception=DestroyExceptionInfo(sans_exception);
+    {
+      ExceptionInfo
+        *sans_exception;
+
+      sans_exception=AcquireExceptionInfo();
+      type_info=GetTypeInfoByFamily((const char *) NULL,draw_info->style,
+        draw_info->stretch,draw_info->weight,sans_exception);
+      if (type_info == (const TypeInfo *) NULL)
+        type_info=GetTypeInfo("*",sans_exception);
+      sans_exception=DestroyExceptionInfo(sans_exception);
+    }
   if (type_info == (const TypeInfo *) NULL)
     {
       status=RenderFreetype(image,draw_info,draw_info->encoding,offset,metrics,
@@ -1290,10 +1288,13 @@ static unsigned long FreetypeReadStream(FT_Stream stream,unsigned long offset,
   FILE *file = (FILE *) stream->descriptor.pointer;
   if (file == (FILE *) NULL)
     return(0);
-  if (count == 0)
-    return(0);
-  if (fseek(file,(off_t) offset,SEEK_SET) != 0)
-    return(0);
+  if (count == 0) /* seek operation */
+    {
+      if (offset > stream->size)
+        return(1);
+
+      return((unsigned long) fseek(file,(off_t) offset,SEEK_SET));
+    }
   return((unsigned long) fread(buffer,1,count,file));
 }
 
@@ -1580,7 +1581,11 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   face_index=(FT_Long) draw_info->face;
   (void) memset(&args,0,sizeof(args));
   if (draw_info->font == (char *) NULL)
-    args.pathname=ConstantString("helvetica");
+    {
+      const TypeInfo *type_info = GetTypeInfo("*",exception);
+      if (type_info != (const TypeInfo *) NULL)
+        args.pathname=ConstantString(type_info->glyphs);
+    }
   else
     if (*draw_info->font != '@')
       args.pathname=ConstantString(draw_info->font);
@@ -1590,7 +1595,8 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
           Extract face index, e.g. @msgothic[1].
         */
         ImageInfo *image_info = AcquireImageInfo();
-        (void) strcpy(image_info->filename,draw_info->font+1);
+        (void) CopyMagickString(image_info->filename,draw_info->font+1,
+          MagickPathExtent);
         (void) SetImageInfo(image_info,0,exception);
         face_index=(FT_Long) image_info->scene;
         args.pathname=ConstantString(image_info->filename);
