@@ -88,6 +88,7 @@
 #include "MagickCore/semaphore.h"
 #include "MagickCore/signature-private.h"
 #include "MagickCore/statistic.h"
+#include "MagickCore/statistic-private.h"
 #include "MagickCore/string_.h"
 #include "MagickCore/thread-private.h"
 #include "MagickCore/timer.h"
@@ -375,7 +376,7 @@ static double ApplyEvaluateOperator(RandomInfo *random_info,const Quantum pixel,
     }
     case PowEvaluateOperator:
     {
-      if (pixel < 0)
+      if ((pixel < 0) && ((value-floor(value)) > MagickEpsilon))
         result=(double) -(QuantumRange*pow((double) -(QuantumScale*pixel),
           (double) value));
       else
@@ -1027,7 +1028,7 @@ static Quantum ApplyFunction(Quantum pixel,const MagickFunction function,
         width;
 
       /*
-        Arcsin (peged at range limits for invalid results): width, center,
+        Arcsin (pegged at range limits for invalid results): width, center,
         range, and bias.
       */
       width=(number_parameters >= 1) ? parameters[0] : 1.0;
@@ -1723,16 +1724,6 @@ MagickExport ChannelMoments *GetImageMoments(const Image *image,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-
-static inline double MagickLog10(const double x)
-{
-#define Log10Epsilon  (1.0e-11)
-
-  if (fabs(x) < Log10Epsilon)
-    return(log10(Log10Epsilon));
-  return(log10(fabs(x)));
-}
-
 MagickExport ChannelPerceptualHash *GetImagePerceptualHash(const Image *image,
   ExceptionInfo *exception)
 {
@@ -2268,8 +2259,8 @@ MagickExport ChannelStatistics *GetImageStatistics(const Image *image,
                 continue;
               }
             median[n++]=p[i];
+            p+=GetPixelChannels(image);
           }
-          p+=GetPixelChannels(image);
         }
         channel_statistics[channel].median=(double) median[
           GetMedianPixel(median,n)];
