@@ -134,11 +134,11 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   list=(char **) AcquireMagickMemory(sizeof(*filelist));
   if (list == (char **) NULL)
@@ -164,11 +164,17 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) CloneString(&read_info->size,DefaultTileGeometry);
   for (i=0; i < (ssize_t) number_files; i++)
   {
-    if (image_info->debug != MagickFalse)
+    char
+      extension[MagickPathExtent];
+
+    if (IsEventLogging() != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"name: %s",
         filelist[i]);
     if (LocaleNCompare(filelist[i],"VID:",4) == 0)
       continue;
+    GetPathComponent(filelist[i],ExtensionPath,extension);
+    if (LocaleNCompare(extension,"VID",3) == 0)
+      continue;
     (void) CopyMagickString(read_info->filename,filelist[i],MagickPathExtent);
     *read_info->magick='\0';
     next_image=ReadImage(read_info,exception);
@@ -182,7 +188,7 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) SetImageProperty(next_image,"label",label,exception);
         label=DestroyString(label);
       }
-    if (image_info->debug != MagickFalse)
+    if (IsEventLogging() != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "geometry: %.20gx%.20g",(double) next_image->columns,(double)
         next_image->rows);
@@ -196,7 +202,7 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         next_image=DestroyImage(next_image);
         next_image=thumbnail_image;
       }
-    if (image_info->debug != MagickFalse)
+    if (IsEventLogging() != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "thumbnail geometry: %.20gx%.20g",(double) next_image->columns,(double)
         next_image->rows);
@@ -216,7 +222,7 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Create the visual image directory.
   */
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
-  if (image_info->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),"creating montage");
   montage_image=MontageImageList(image_info,montage_info,
     GetFirstImageInList(images),exception);
@@ -315,6 +321,9 @@ ModuleExport void UnregisterVIDImage(void)
 static MagickBooleanType WriteVIDImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
+  const MagickInfo
+    *magick_info;
+
   Image
     *montage_image;
 
@@ -345,8 +354,9 @@ static MagickBooleanType WriteVIDImage(const ImageInfo *image_info,Image *image,
   write_info=CloneImageInfo(image_info);
   *write_info->magick='\0';
   (void) SetImageInfo(write_info,1,exception);
-  if ((*write_info->magick == '\0') ||
-      (LocaleCompare(write_info->magick,"VID") == 0))
+  magick_info=GetMagickInfo(write_info->magick,exception);
+  if ((magick_info == (const MagickInfo*) NULL) ||
+      (LocaleCompare(magick_info->module,"VID") == 0))
     (void) FormatLocaleString(montage_image->filename,MagickPathExtent,
       "miff:%s",write_info->filename);
   status=WriteImage(write_info,montage_image,exception);
